#ifndef BASIN_HH_
#define BASIN_HH_

#include "ell/State.hh"
#include "ell/StateCollector.hh" 

namespace ell
{


	/*!
	 * Wrapper class around the basin algorithm. For a detailed description see
	 * static member function Basin::basin(..).
	 * 
	 */
	class Basin {
	protected:
		  //! the maximal energy all states should be below
		const double maxE;
		  //! the delta E to lower maxE in case too many states are in the queue
		const double deltaE;
		  //! the maximal number of elements the underlying queue is allowed to
		  //! contain
		const size_t maxToStore;
		
	public:
		
		  //! the updatable value stored for a key
		class QueueVal {
		public:
		
			  //! the smallest neighbor seen so far
			CSequence minNeigh;
			  //! the energy of minNeigh
			double minNeighE;
			  //! index of the minimum that is the end of the gradient walk 
			  //! starting in minNeigh (the smalles neighbor seen)
			size_t minNeighGWE;
			
			QueueVal();
			~QueueVal();
		};

		
	public:
		
		  //! Construction of the wrapper.
		  //!
		  //! @param maxE the maximal energy all states should be below
		  //! @param deltaE the delta E to lower maxE in case too many states are 
		  //!               in the queue
		  //! @param maxToStore the maximal number of elements the underlying queue
		  //!                   is allowed to contain
		Basin(	const double maxE, 
				const double deltaE, 
				const size_t maxToStore 
				);
		
		  //! Calls the static member function Basin::basin(..) with the 
		  //! parameters from construction to flood the gradient basin of  
		  //! localMin, storing all States of the basin in sc.
		  //!
		  //! @param localMin the local minimum the basin belongs to
		  //! @param scBasin the StateCollector all states that belong to the  
		  //!           gradient basin are written to
		  //! @param scSurface the StateCollector all states that form the 
		  //!           surface of the basin (are not part of it) are written to
		  //! @return the maximal energy until the basin was flooded (all accepted 
		  //!         state have energy below)  
		double flood(	const State* const localMin,
						StateCollector* scBasin,
						StateCollector* scSurface = NULL);
		
		
		  //! Fills succesivelly the basin of the given local minimum in the 
		  //! landscape and stores the seen States in energy increasing order to
		  //! the given StateCollector.
		  //! The basin is flooded up to maxE.
		  //! Therefore, all neighbored States with energy below maxE have to be 
		  //! stored in a queue. A maximal number of states to store
		  //! for investigation in parallel can be given by maxToStore. If the 
		  //! underlying queue size exeeds this maxE is decreased for deltaE and 
		  //! all States above the new maxE are removed from the queue. This is 
		  //! repeated until the queue size falls back below maxToStore.
		  //!
		  //! @param localMin the local minimum the basin belongs to
		  //! @param scBasin the StateCollector all states that belong to the  
		  //!           gradient basin are written to
		  //! @param maxE the maximal energy all states should be below
		  //! @param deltaE the delta E to lower maxE in case too many states are 
		  //!               in the queue
		  //! @param maxToStore the maximal number of elements the underlying queue
		  //!                   is allowed to contain
		  //! @param scSurface the StateCollector all states that form the 
		  //!           surface of the basin (are not part of it) are written to
		  //! @return the maximal energy until the basin was flooded (all accepted 
		  //!         state have energy below)  
		static double basin(	const State* const localMin, 
								StateCollector* scBasin,
								const double maxE, 
								const double deltaE, 
								const size_t maxToStore, 
								StateCollector* scSurface = NULL
							);
	};
	

}

#endif /*BASIN_HH_*/
