#ifndef WALKABORTIONCRITERION_HH_
#define WALKABORTIONCRITERION_HH_


#include "ell/StateCollector.hh"

namespace ell
{
	/*!
	 * This abstract class is used to check if a walk should be 
	 * finished based on properties extracted from a StateCollector.
	 * 
	 * @author Daniel Maticzka
	 */
	class WalkAbortionCriterion
	{
	public:
		
		virtual ~WalkAbortionCriterion(){};
		
		/*!
		 * Returns true if the StateCollector sc fulfills a defined property.
		 */
		virtual bool abort(StateCollector* sc) const = 0;
	};
	
	
	/*!
	 * This subclass of WalkAbortionCriterion aborts a walk if the
	 * number of accepted States exceeds a predefined boundary.
	 * 
	 * @author Daniel Maticzka
	 */
	class WAC_MaxLength : public WalkAbortionCriterion
	{
	private:
		//! maximum number of accepted states
		const size_t maxStates;	
		
	public:
		/*!
		 * Constructs a WalkAbortionCriterion that aborts when a maximum
		 * number of accepted states is reached.
		 * @param max maximum number of accepted states
		 */
		WAC_MaxLength(const size_t max);
		bool abort(StateCollector* sc) const;
	};
	
	
	/*!
	 * This subclass of WalkAbortionCriterion aborts a walk if the
	 * energy of the last accepted State exceeds a predefined boundary.
	 * 
	 * @author Daniel Maticzka
	 */
	class WAC_MaxEnergy : public WalkAbortionCriterion
	{
	private:

		//! energy boundary that won't be exceeded
		const double maxEnergy;
		
	public:
		/*!
		 * Constructs a WalkAbortionCriterion that aborts a walk if a maximum
		 * energy is exceeded.
		 * @param max the energy to check against
		 */
		WAC_MaxEnergy(const double max);
	 	bool abort(StateCollector* sc) const;
	};	
	
	
	/*!
	 * This subclass of WalkAbortionCriterion aborts a walk if the
	 * energy of the last accepted State falls below a predefined boundary.
	 * 
	 * @author Daniel Maticzka
	 */
	class WAC_MinEnergy : public WalkAbortionCriterion
	{
	private:
		//! minimal allowed energy
		const double minEnergy;
		
	public:
		/*!
		 * Constructs a WalkAbortionCriterion that aborts a walk if a the energy
		 * of an accepted State is below the minimum boundary.
		 * @param min the energy to check against
		 */
	 	WAC_MinEnergy(const double min);
	 	bool abort(StateCollector* sc) const;
	};
	
	/*!
	 * This subclass of WalkAbortionCriterion never aborts a walk.
	 * 
	 * @author Daniel Maticzka
	 */
	class WAC_OpenEnd : public WalkAbortionCriterion
	{
	public:
	 	WAC_OpenEnd();
	 	bool abort(StateCollector* sc) const;
	};
	
	/*!
	 * This subclass sof WalkAbortionCriterion aborts a walk if the
	 * one of the given WalkAbortionCriterion objects aborts.
	 * 
	 * @author Daniel Maticzka
	 */
	class WAC_OR : public WalkAbortionCriterion
	{
	private:
		const WalkAbortionCriterion& wac_a;
		const WalkAbortionCriterion& wac_b;
		
	public:
		/*!
		 * @param wac_a first WalkAbortionCriterion to check
		 * @param wac_b second WalkAbortionCriterion to check
		 */
		WAC_OR(	const WalkAbortionCriterion& wac_a, 
				const WalkAbortionCriterion& wac_b);
		bool abort(StateCollector* sc) const;
	};
		
} // namespace ell

#endif /*WALKABORTIONCRITERION_HH_*/
