#include "ell/Walk.hh"
#include "ell/State.hh"
#include "ell/StateCollector.hh"
#include "ell/SC_Listing.hh"
#include "ell/StateAcceptor.hh"
#include <iostream>
#include <string>
#include <vector>
#include <stdlib.h>
#include <math.h>
#include "biu/RandomNumberGenerator.hh"
#include "biu/RandomNumberFactory.hh"

using namespace ell;

/*!
 * This is a dummy implementation of a State class including inner classes
 * NeighborList and ItState. It has a limited implementation of the functions
 * getRandomNeighbor, getNeighborList and getRandomNeighborList.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
public:
	class NeighborList : public State::NeighborList 
	{
	private:
		std::string id;
		double origin;
	public:
		class ItState : public State::NeighborList::ItState {
		public:
			int pos;
			
			ItState():pos(0) {}
			
			ItState(const ItState &itstate) {pos=itstate.pos;}
			
			virtual ItState * clone() const {
				return new ItState(*this);
			}
		};
    
		NeighborList(std::string _id, double _origin) 
		{
			id = _id;
			// first successor starts with relative energy +3
			origin = _origin + 4;
		}
    
		virtual ~NeighborList() {}
  
		virtual State*
		next(State::NeighborList::ItState *itstate, State* elem) const {
			((DummyState*)elem)->applyMove();
			return elem;
		}

		virtual State* 
		first(State::NeighborList::ItState** itstate) const {
			DummyState* s = new DummyState(id, origin);
			s->applyMove();
			return s;
		}
	};
	
	std::string id;
	double energy;
	double maxEnergy;

	DummyState(std::string _id, double _energy) 
		{ id = _id; energy = _energy;} 
	~DummyState() {}
	
	bool operator== (const State& state2) const { return true; }
	bool operator!= (const State& state2) const { return true; }

	int	getFitness() const { return 0; }
	double getEnergy() const { return energy; }
	unsigned int getMinimalDistance(const State& state2) const { return 0; }
	State* clone() const { return new DummyState(id, energy); }
	State* fromString(const std::string& stringRep) const { return NULL; }
	std::string	toString() const { return id; }
	
	NeighborListPtr getNeighborList() const 
	{ 
		return NeighborListPtr(new NeighborList(id,energy));
	}
	
	NeighborListPtr getRandomNeighborList() const {
		return NeighborListPtr(new NeighborList(id,energy+5));
	}
	
	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const 
	{ 
		int randomModifier = (int) ((((double) biu::RNF::getRN()) / ((double) biu::RNF::getMaxRN()))*10)-5;
		return new DummyState(id,energy + randomModifier); 
	}
	
	DummyState& applyMove() {
		// decrease energy by one
		energy -= 1;
		return *this;
	}
	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
	CSequence  compress(void) const {
		return CSequence();
	}
	
	CSequence&   compress(CSequence& toFill) const {
		toFill.clear();
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq, State* toFill) const {
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq) {
		return NULL;
	}
};


int main(int argc, char **argv) {
	
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : walkMC\n"
				<<"===========================================" <<std::endl;
	
		// init randomizer	
	std::cout	<<"=> RND::setRNG( biu::RNG_LCG() )" <<std::endl;
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG( rng );
	delete rng;
	const unsigned int seed = 1;
	std::cout	<<"=> biu::RNF::setSeed("<<seed<<")" <<std::endl;
	biu::RNF::getRNG().setSeed(seed);
	
	std::cout	<< "==> testing RNG" << std::endl;
	for (int i=1; i<=10; i++){
	std::cout 	<< i << ": " << (((double) biu::RNF::getRN()) / ((double) biu::RNF::getMaxRN())) << std::endl;
	}
	
	std::cout	<< "=> evaluation of used metropolis criterion:" << std::endl;
	std::cout	<< "exp((-1)* -5 ) = " << exp((-1)* -5 ) << std::endl;
	std::cout	<< "exp((-1)* -4 ) = " << exp((-1)* -4 ) << std::endl;
	std::cout	<< "exp((-1)* -3 ) = " << exp((-1)* -3 ) << std::endl;
	std::cout	<< "exp((-1)* -2 ) = " << exp((-1)* -2 ) << std::endl;
	std::cout	<< "exp((-1)* -1 ) = " << exp((-1)* -1 ) << std::endl;
	std::cout	<< "exp((-1)* 0 ) = " << exp((-1)* 0 ) << std::endl;
	std::cout	<< "exp((-1)* 1 ) = " << exp((-1)* 1 ) << std::endl;
	std::cout	<< "exp((-1)* 2 ) = " << exp((-1)* 2 ) << std::endl;
	std::cout	<< "exp((-1)* 3 ) = " << exp((-1)* 3 ) << std::endl;
	std::cout	<< "exp((-1)* 4 ) = " << exp((-1)* 4 ) << std::endl;
	std::cout	<< "exp((-1)* 5 ) = " << exp((-1)* 5 ) << std::endl;


	std::cout	<< "=> creating SA_Metropolis" << std::endl;
	StateCollector* sc_pre = new SC_Listing();
	double energy = 20;
	State& state_pre = *(new DummyState("SC_State", energy));
	sc_pre->add(state_pre);
	energy = 21;
	State& state_succ = *(new DummyState("state 1", energy));
	StateAcceptor* sa = new SA_Metropolis(1);
	for (int i=1; i<=10; i++){
		std::cout 	<< "->accept(old energy: 20, successor energy: 21): " 
					<< sa->accept(sc_pre, state_succ) 
					<< std::endl;
	}
	delete &state_pre;
	delete &state_succ;
	delete sc_pre;
	delete sa;
	
	std::cout	<< "=> creating DummyState" << std::endl;
	energy = 20;
	State* state = new DummyState("state 1", energy);
	std::cout	<< "->getEnergy = " << state->getEnergy() << std::endl;
	std::cout	<< "=> printing 5 successors:" << std::endl;
	State::NeighborListPtr neighbors = state->getNeighborList();
	State::NeighborList::Iterator it = neighbors->begin();
	for (int i=1; i<5; i++) {
		std::cout	<< i << ". successor: " << (*it).getEnergy() << std::endl;
		++it;
	}
	
	std::cout	<< "=> creating StateCollector" << std::endl;
	StateCollector* sc = new SC_Listing();
	std::cout	<< "=> calling walk" << std::endl;
	double beta = 2;
	int maxstates = 10;
	WalkAbortionCriterion* wac = new WAC_MaxLength(maxstates);
	sc = WalkMC::walkMC(state, sc, wac, beta);
	
	std::cout 	<< "Number Of Accepted States: " 
					<< sc->size()
					<< std::endl;
	
	SC_Listing* sch;
	sch = dynamic_cast<SC_Listing*>(sc);
	
	std::cout 	<< "Accepted States:" << std::endl;
	SC_Listing::List list = sch->getList();
	SC_Listing::List::iterator iter;
  	for( iter = list.begin(); iter != list.end(); iter++ ) {
    	std::cout 	<< "  * Energy = "
    				<< (*iter)->getEnergy() << std::endl;
  	}

  	delete state;
  	delete sc;
  	delete wac;
  	
  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
  	
	return 0; 
}
